;	WPAGE.CMD:	Wordprocessing Macro Page
;			for MicroEMACS 3.13 and above
;			(C)opyright 1987 by Daniel M Lawrence
;
;	Modifications by Thierry Godefroy:
;
;	Added right justifying, QTyp II spell checking and printing macros.

; set the clean procedure up
store-procedure clean
	delete-buffer "[w-center]"
	delete-buffer "[w-scheck]
	delete-buffer "[w-indformat]"
	delete-buffer "[w-notready]"
	delete-buffer "[w-swapword]"
	delete-buffer "[w-justify]"
	delete-buffer "[w-unjustify]"
	unbind-key S-FN3
	unbind-key S-FN9

	set $cmode %oldmode
	set $gmode %oldgmode
!endm

; make sure the function key window is up
	set %rcfkeys FALSE
	toggle-fkeys
	write-message "Loading..."

; Write out the page instructions
	save-window
	1 next-window
	beginning-of-file
	set $curcol 25
	overwrite-string " F1 center line            F2 right justify paragr. "
	next-line
	set $curcol 25
	overwrite-string " F3 reformat paragraph     F4 left justify region   "
	next-line
	set $curcol 25
	overwrite-string " F5 ref indented paragraph F6 spell check text      "
	next-line
	set $curcol 18
	overwrite-string "WORD"
	set $curcol 25
	overwrite-string " F7 toggle words           F8 print current buffer  "
	next-line
	set $curcol 25
	overwrite-string " F9 count words           F10 printer & page setup  "
	unmark-buffer
	beginning-of-file
	!force restore-window
	update-screen

; Center the current line

store-procedure w-center
	end-of-line
	set %rctmp $curcol
	beginning-of-line
	&div &sub $fillcol %rctmp 2 insert-space
	beginning-of-line
!endm

; Justify the current paragraph

store-procedure w-justify
	set $discmd FALSE
	*nxtlin
	set %herel $curline
	; Check if there is something in next line
	next-line
	beginning-of-line
	set %tmp1 $curcol
	end-of-line
	set %tmp2 $curcol
	goto-line %herel
	!if &equ %tmp1 %tmp2
		!goto justdone
	!endif
	; Remove trailing spaces
	beginning-of-line
	set-mark
	write-message "Justifying..."
	end-of-line
	trim-region
	goto-line %herel
	; skip leading spaces and control characters
	beginning-of-line
	!while &less $curchar 33
		forward-character
	!endwhile
	; Count gaps on line
	set %gaps 0
	!while TRUE
		search-forward " "
		!if &not &equ %herel $curline
			!break
		!endif
		!while &equ $curchar 32
			delete-next-character
		!endwhile
		set %gaps &add %gaps 1
	!endwhile
	goto-line %herel
	; Skip just'n if gaps less than one
	!if &less %gaps 1
		!goto linedone
	!endif
	; Calculate constants for justification
	end-of-line
	set %2n &sub $fillcol $curcol
	set %2n &add %2n %2n
	set %g %gaps
	set %2g &add %g %g
	set %s 0
	; For each gap
	!while &greater %gaps 0
		search-reverse " "
		; Pad out this gap
		set %s &add %s %2n
		!while &greater %s %g
			insert-space
			set %s &sub %s %2g
		!endwhile
		set %gaps &sub %gaps 1
	!endwhile
	;
	*linedone
	; Next line
	next-line
	!goto nxtlin
	*justdone
	next-line
	$hilight remove-mark
	clear-message-line
	set $discmd TRUE
!endm

; Un-justify a region

store-procedure w-unjustify
	set $discmd FALSE
	set %lastl $curline
	; Make sure %lastl is not before %firstl
	!force exchange-point-and-mark
	!if &gre $curline %lastl
		set %firstl %lastl
		set %ladtl $curline
		!force exchange-point-and-mark
	!else
		set %firstl $curline
	!endif
	write-message &cat &cat &cat "left justify lines " %firstl " to " %lastl
	set %lastl &add %lastl 1	; Make sure last line included
	beginning-of-line
	!while &gre %lastl $curline
		; go to next gap between words
		!force search-forward " "
		!if &or &seq $status FALSE &less %lastl $curline
			!break
		!endif
		; Remove surplus spaces
		!while &equ $curchar 32
			delete-next-character
		!endwhile
	!endwhile
	$hilight remove-mark
	set $discmd TRUE
	clear-message-line
!endm

;	reformat indented paragraph

store-procedure w-indformat
	write-message "                    [Fixing paragraph]"

	;remember where we are..and set up
	set %endpos $cwline
	set $discmd "FALSE"

	;snarf the paragraph and put it in its own buffer
	kill-region
	2 split-current-window
	select-buffer "[temp]"
	yank

	;get rid of the blank lines at the end
	end-of-file
	!while &and &seq $line "" &not &equ $curline 1
		delete-previous-character
	!endwhile
	forward-character

	;get rid of the indents
	set %endline $curline
	beginning-of-file
	!while &gre %endline $curline
		!while &or &equ $curchar 32 &equ $curchar 9
			delete-next-character
		!endwhile
		next-line
	!endwhile

	;re-format it
	&sub $fillcol 8 set $fillcol
	1 goto-line
	fill-paragraph
	&add $fillcol 8 set $fillcol

	;insert new indents
	end-of-file
	set %endline $curline
	beginning-of-file
	!while &less $curline &sub %endline 1
		beginning-of-line
		handle-tab
		next-line
	!endwhile

	;move it back into the original buffer
	beginning-of-file
	set-mark
	end-of-file
	kill-region
	unmark-buffer
	delete-window
	yank

	;and clean up
	delete-buffer "[temp]"
	set $cwline %endpos
	set $discmd TRUE
	write-message "                    [Fixed paragraph]"
!endm

;	swap the word the cursor is on and it's predisesor

store-procedure w-swapword
	set %rctmp $search
	forward-character
	previous-word
	0 delete-next-word
	set %second $kill
	insert-string "~~"
	!force previous-word
	!if &seq $status FALSE
		set %first %second
	!else
		0 delete-next-word
		set %first $kill
		insert-string %second
	!endif
	search-forward "~~"
	delete-previous-character
	insert-string %first
	previous-word
	set $search %rctmp
!endm

store-procedure print-text
	qmenu-read-string "Page header" "Document title (nothing if no header)" $title
	set $title $rval
	qmenu-item-select "Page numbering" "Should the pages be numbered ?" "Yes" "No"
	set $pageflag &seq $rval 1
	print-buffer
!endm

store-procedure printer-setup
	qmenu-read-string "Page length" "Page length (in lines)" $pagesize
	set $pagesize $rval
	qmenu-read-string "Printer port" "Printer port device:" $printer
	set $printer $rval
	qmenu-item-select "Character translation" "Enable character translation ?" "Yes" "No"
	set $translat &seq $rval 1
!endm

macro-to-key w-center S-FN1
macro-to-key w-justify S-FN2
bind-to-key fill-paragraph S-FN3
macro-to-key w-unjustify S-FN4
macro-to-key w-indformat S-FN5
bind-to-key spell-check S-FN6
macro-to-key w-swapword S-FN7
macro-to-key print-text S-FN8
bind-to-key count-words S-FN9
macro-to-key printer-setup S-FN0

set %oldmode $cmode
set %oldgmode $gmode
add-mode wrap
add-global-mode wrap
write-message "[Wordprocessing mode loaded]"
